﻿using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc.Controllers;
using Microsoft.OpenApi.Models;
using Swashbuckle.AspNetCore.SwaggerGen;
using System.Linq;
using System.Reflection;

namespace Mirea.Api.Endpoint.Configuration.SwaggerOptions;

public class TagSchemeFilter : IOperationFilter
{
    public void Apply(OpenApiOperation operation, OperationFilterContext context)
    {
        if (context.ApiDescription.ActionDescriptor is not ControllerActionDescriptor controllerActionDescriptor)
            return;

        var controllerType = controllerActionDescriptor.ControllerTypeInfo;

        var tagsAttribute = controllerType.GetCustomAttributes<TagsAttribute>(inherit: true).FirstOrDefault();

        if (tagsAttribute == null)
        {
            var baseType = controllerType.BaseType;
            while (baseType != null)
            {
                tagsAttribute = baseType.GetCustomAttributes<TagsAttribute>(inherit: true).FirstOrDefault();
                if (tagsAttribute != null)
                    break;

                baseType = baseType.BaseType;
            }
        }

        if (tagsAttribute == null)
            return;

        operation.Tags ??= [];
        operation.Tags.Add(new OpenApiTag { Name = tagsAttribute.Tags[0] });
    }
}