﻿using MediatR;
using Microsoft.EntityFrameworkCore;
using Mirea.Api.DataAccess.Application.Common.Exceptions;
using Mirea.Api.DataAccess.Application.Cqrs.Professor.Queries.GetProfessorDetails;
using Mirea.Api.DataAccess.Application.Interfaces.DbContexts.Schedule;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;

namespace Mirea.Api.DataAccess.Application.Cqrs.Professor.Queries.GetProfessorDetailsBySearch;

public class GetProfessorInfoSearchQueryHandler(IProfessorDbContext dbContext) : IRequestHandler<GetProfessorInfoSearchQuery, ProfessorInfoListVm>
{
    public async Task<ProfessorInfoListVm> Handle(GetProfessorInfoSearchQuery request, CancellationToken cancellationToken)
    {
        var name = request.Name.ToLower();
        var professor = await dbContext.Professors
                        .Where(p => p.Name.ToLower().Contains(name) ||
                                    (p.AltName != null && p.AltName.ToLower().Contains(name)))
                        .ToListAsync(cancellationToken)
                    ?? throw new NotFoundException(typeof(Domain.Schedule.Professor), request.Name);

        return new ProfessorInfoListVm()
        {
            Details = professor.Select(x => new ProfessorInfoVm()
            {
                Id = x.Id,
                Name = x.Name,
                AltName = x.AltName
            })
        };
    }
}